/*

ViruNetSim
 
Dit is een heel simpel Monte-Carlo graafmodelletje (netwerkmodel) om het effect
van contactbeperking op de verspreiding van een virus te simuleren. Het is niet
bedoeld voor serieus gebruik.

Natuurlijk had dit wiskundig berekend kunnen worden, met de graventheorie.
Ma(c) Evert Mouw (2021)\n");ar ik heb toch een simulatie gemaakt, en wel hierom:

1. Dat vind ik leuker.
2. Ik ben slecht in de wiskunde van de graventheorie.
3. Een simulatie is uitbreidbaar en te visualiseren. Je kunt er leuk mee spelen.

Bovendien zijn de meeste modellaties al gebaseerd op zulke modellen met formules.
Een simpele Monte-Carlo simulatie heeft ook wel charme.

Soms loopt ie een beetje vast met het zoeken naar verbindingen; opnieuw starten 
geeft een andere srand waarde en kan dan helpen. Bij mijn laatste runs had ik er 
geen last meer van.

De parameters kun je wijzigen door de constanten te veranderen en te hercompileren.

gcc -Wall -lm virusimnet.c -o virusimnet

Grafiekje maken met Graphviz:
 
dot -Tpng virusimnet.dot -o virusimnet.png

SUGGESTIES VOOR VERBETERING:

- maximaal aantal contacten is een beperking; kan omgebouwd worden van array naar linkedlist
- er is nu geen dealloc (maar nou en, ik sluit toch af na de berekening)
- resultaten zijn nu na 1 dag; graag na 1 week of een in te stellen cyclus
- ook andere effecten meeberekenen, zoals afstand houden e.d. ??
- ?? edges apart in een linked list bewaren
- ?? ?? geen pointers en structs, maar arrays (minder gelijkvormigheid in code, wel sneller)

Evert Mouw <post@evert.net>
januari 2021
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

// constanten
#define MAXCONTACTEN 20
#define AANTAL 5000
#define NDAILY 14
#define R0 2.2
#define B 0.000261
#define HERHAAL 1
#define BOOST 100
#define DOTFILE "virusimnet.dot"
#define TSVFILE "virusimnet.tsv"

// datastructuur
// graaf met personen die onderling contact hebben
struct Personen {
	struct Personen * contacten[MAXCONTACTEN]; // graaf
	int besmet;
	int rekenstatus; // interne variabele om bij te houden of ie al berekend is
};

// globals
struct Personen * eerste = NULL;
struct Personen * lijst[AANTAL];
int verbonden[AANTAL][AANTAL];

// prototypes nodig voor het compileren: hier niet nodig

int sanity () {
	if ( MAXCONTACTEN > AANTAL ) return 1;
	if ( NDAILY > AANTAL ) return 1;
	if ( NDAILY > MAXCONTACTEN ) return 1;
	if ( R0 < 0 ) return 1;
	if ( B < 0 ) return 1;
	if ( MAXCONTACTEN < 0 ) return 1;
	if ( AANTAL < 0 ) return 1;
	if ( NDAILY < 0 ) return 1;
	return 0;
}

// maak de graaf pointerstructuur
void maakgraaf () {
	struct Personen * huidige = NULL;
	if (eerste != NULL) return;
	int i;
	for (i=0; i<AANTAL; i++) {
		// C is best, maar C++ met "new Personen" is hier toch wel fijner
		huidige = (struct Personen*)malloc(sizeof(struct Personen));
		lijst[i] = huidige;
	}
	eerste = lijst[0];
}

// initialiseer de graaf met beginwaarden
void initgraaf () {
	int i,j;
	for (i=0; i<AANTAL; i++) {
		lijst[i]->besmet = 0;
		lijst[i]->rekenstatus = 0;
		for (j=0; j<MAXCONTACTEN; j++) {
			lijst[i]->contacten[j] = NULL;
		}
	}
}

// maak semi-willekeurig verbindingen
// - gemiddeld NDAILY (ca. 14 contacten) per dag
// - elk contact is een verbinding
// - niet alle nodes hebben evenveel verbindingen
// - de helft van de contacten zit "in de buurt"
// NOTE: afhankelijk van de instellingen (define) kan dit lang duren of zelfs vastlopen...
void verbind () {
	// x ~ willekeurige node 1
	// y ~ willekeurige node 2
	// verbind x en y wederzijds
	// doelpercentage: 100% is maximaal onderling verbinden
	int i,j;
	int x,y;
	int cumulatief;
	int mislukkingen;
	// resetten geheugen van verbindingen
	for (i=0; i<AANTAL; i++) {
		for (j=0; j<AANTAL; j++) {
			verbonden[i][j] = 0;
		}
	}
	for (i=0; i<AANTAL; i++) {
		int doelaantal = rand() % (NDAILY*2); // gemiddeld NDAILY
		if (doelaantal==0) doelaantal++; // minstens een enkele verbinding
		if (doelaantal>MAXCONTACTEN) doelaantal = MAXCONTACTEN;
		cumulatief = 0;
		mislukkingen = 0;
		const int buurt = 6;
		int gok;
		while (cumulatief < doelaantal) {
			//printf("i = %i, doelaantal = %i, cumulatief = %i\n",i,doelaantal,cumulatief);
			mislukkingen++;
			if ( mislukkingen > (AANTAL/2) ) break; // TODO min of meer willekeurig gekozen breakoff waarde
			x = i;
			// de helft van de verbindingen moet in de buurt zitten
			gok = rand() % 100;
			if ( gok < 50 && x > buurt ) {
				y = x - ( rand() % buurt );
			} else {
				// en anders een volledig willekeurig contact zoeken
				y = rand() % AANTAL;
			}
			// niet met jezelf verbinden
			if (x == y) continue;
			// als de target node al vol zit met contacten, dan opnieuw
			if ( lijst[y]->contacten[MAXCONTACTEN-1] != NULL ) continue; //while
			// als ze al verbonden zijn, dan ook opnieuw proberen
			if ( verbonden[x][y] || verbonden[y][x] ) continue; //while
			//
			// mooi, we zijn door de checks heen :-)
			// maak x-->y
			j = 0;
			while (j < MAXCONTACTEN) {
				if (lijst[x]->contacten[j] == NULL) {
					lijst[x]->contacten[j] = lijst[y];
					verbonden[x][y] = 1;
					mislukkingen = 0;
					break; //while
				} else {
					// er was al een contact; bij het cumulatief aantal contacten zetten
					cumulatief++;
				}
				j++;
			}
			// maak y-->x
			j = 0;
			while (j < MAXCONTACTEN) {
				if (lijst[y]->contacten[j] == NULL) {
					lijst[y]->contacten[j] = lijst[x];
					verbonden[y][x] = 1;
					mislukkingen = 0;
					break;
				}
				j++;
			}
			cumulatief++;
		}
	}
}

// loop door de graaf en knip willekeurig verbindingen weg
// (reduceer naar een doelpercentage)
void reduceer (int percentage) {
	int i,j;
	int z,t;
	for (z=0; z<AANTAL; z++) {
		i = 0;
		// wegknippen per contact van deze persoon "zomaar"
		while (i < MAXCONTACTEN) {
			// alleen doorgaan als een random percentage
			// boven het doelpercentage zit 
			int r = rand() % 100;
			if (r < percentage) break; //while
			//
			if (lijst[z]->contacten[i] != NULL) {
				// tegenpartij van het contact achterhalen
				struct Personen * tegenpartij = lijst[z]->contacten[i];
				for (j=0; j<MAXCONTACTEN; j++) {
					if ( tegenpartij->contacten[j] == lijst[z] ) {
						tegenpartij->contacten[j] = NULL;
						break; //for
					}
				}
				t = 0;
				for (j=0; j<MAXCONTACTEN; j++) {
					if ( lijst[j] == tegenpartij ) {
						t = j;
						break; //for
					}
				}
				verbonden[t][z] = 0;
				//
				// en nu het eigen contact wegknippen
				lijst[z]->contacten[i] = NULL;
				verbonden[z][t] = 0;
			} else {
				// er zijn geen contacten meer voor deze persoon
				break; //while
			}
			i++;
		}
	}
}

// besmettingsfunctie (recursief) (hulpfunctie)
// (het is de bedoeling dat een andere functie deze hulpfunctie
// aanroept voor alle nodes afzonderlijk; steeds resetten met
// besmet_reset() )
void besmet_hulp (struct Personen * beginpersoon) {
	int j;
	int gok;
	float besmettingskans;
	// int n, d;
	struct Personen * buur;
	if ( beginpersoon == NULL ) return;
	if ( beginpersoon->rekenstatus == 1 ) return;
	beginpersoon->rekenstatus = 1;
	// deze persoon maakt een _kans_ om besmet te raken
	//
	// R0 = b * n / d
	// | b = besmettingskans per contact
	// | n = aantal contacten per dag
	// | d = duratie / genezingstijd <-- tijd van besmettelijkheid
	//
	// b = R0 / ( n / d ) // probability of transmission per contact
	// n = 14;            // contacts per day (NDAILY)
	// d = 8;             // gemiddeld aantal dagen tot antilichamen bij mild verloop; bij ernstig verloop drie weken minstens
	//
	// b ~ COVID-19 infection transmission is 1 infection per 3836 unprotected
	// b ~ Probability of transmission per contact
	// b = 1/3836; // 0.00026068821689259646
	// b = 0.00026068821689259646;
	//
	// bij R=1 raakt 1 persoon van de gemiddeld 14 (NDAILY) contacten besmet
	// dus kans besmetting is R/NDAILY
	// hoe groter R, hoe groter de kans op een treffer binnen [0,NDAILY]
	//
	// TODO sterk verbeteren, dit is een beetje uit de lucht gegrepen
	//
	// voor de sim: booster BOOST zorgt voor snellere besmetting
	// besmettingskans = R0 / (n / d);
	// besmettingskans = 2.1011E-8 * NDAILY;
	// besmettingskans = B * NDAILY;
	besmettingskans = B * NDAILY * BOOST;
	// gokken op een schaal van 0..100; besmettingskans dus ook als percentage (maal honderd)
	gok = rand() % 100;
	//printf ("besmettingskans %f; gok %i\n", besmettingskans*100, gok);
	if (gok < besmettingskans * 100) {
		beginpersoon->besmet = 1;
	} else {
		// niet besmet? dan ook niet verder besmetten!
		return;
	}
	// loop alle "buren" (neighbours) oftewel contacten langs
	for (j=0; j<MAXCONTACTEN; j++) {
		buur = beginpersoon->contacten[j];
		besmet_hulp (buur);
	}
}

// hulpfunctie
void besmet_reset () {
	int i;
	for (i=0; i<AANTAL; i++) {
		lijst[i]->besmet = 0;
		lijst[i]->rekenstatus = 0;
	}
}

// tel het aantal besmette mensen
int aantalbesmet() {
	int i;
	int totaal = 0;
	for (i=0; i<AANTAL; i++) {
		if ( lijst[i]->besmet == 1 ) totaal++;
	}
	return totaal;
}

// simuleer hoe snel een virus propageert
// vanaf elke node (persoon) in de graaf
// en neem de gemiddelde besmettingsgroei
int groeisnelheid () {
	int i;
	int cumulatief = 0;
	int gemiddelde;
	for (i=0; i<AANTAL; i++) {
		besmet_reset();
		besmet_hulp (lijst[i]);
		cumulatief += aantalbesmet();
	}
	gemiddelde = round ( (float)cumulatief / (float)AANTAL );
	return gemiddelde;
}

// export naar een DOT (Graphviz) bestand
void dot() {
	FILE * fp;
	fp = fopen (DOTFILE,"w");
	int i,j;
	int z;
	char c;
	char color_red[]="red";
	char color_green[]="green";
	char * color = NULL;
	fprintf (fp, "strict graph {\n");
	fprintf (fp, "\n");
	// nodes
	for (i=0; i<AANTAL; i++) {
		if ( lijst[i]->besmet == 1 ) {
			c='X';
			color = color_red;
		} else {
			c='~';
			color = color_green;
		}
		fprintf (fp, "%i [label=\"%c #%i\", style=filled, fillcolor=%s]\n", i, c, i, color);
	}
	fprintf (fp, "\n");
	// edges
	for (i=0; i<AANTAL; i++) {
		for (j=0; j<MAXCONTACTEN; j++) {
			if ( lijst[i]->contacten[j] != NULL ) {
				// het pointeradres van "target" opzoeken in de pointer array lijst[]
				z = 0;
				while ( lijst[z] != lijst[i]->contacten[j] ) {
					z++;
				}
				fprintf (fp, "%i -- %i\n", i, z);
			}
		}
	}
	fprintf (fp, "\n");
	fprintf (fp, "}\n");
	fclose (fp);
}

int main() {
	int contactpercentage;
	int besmettingsaantal;
	int herhaal;

	printf ("ViruSimNet Monte-Carlo graafmodelletje (netwerk)\n");
	printf ("om het effect van contactbeperking te simuleren\n");
	printf ("op de verspreiding van een virus. Niet realistisch!\n");
	printf ("(c) Evert Mouw (2021)\n");
	printf ("\n");
	printf ("%i personen in totaal.\n", AANTAL);
	printf ("%i gemiddeld aantal dagelijkse contacten (zonder contactbeperking).\n", NDAILY);
	printf ("%i contacten maximaal per persoon.\n", MAXCONTACTEN);
	printf ("De R0 is %f en b is %f.\n", R0, B);
	printf ("\n");
	printf ("Uitrekenen. Geduld...\n");
	printf ("\n");

	FILE * fp;
	fp = fopen (TSVFILE,"w");
	fprintf (fp, "aantal\tcontact_p\tbesmet_n\n");
	printf (">\taantal\tcontact\tbesmet\n");
	printf (">\t[n]\t[%%]\t[n]\n");

	// initialiseer randomizer op tijd (granulariteit: seconden)
	srand(time(0));

	maakgraaf();

	for (herhaal=0; herhaal<HERHAAL; herhaal++) {

		for ( contactpercentage = 10; contactpercentage <= 100; contactpercentage += 10) {

			initgraaf();
			//printf ("#%i %i%% Verbinden...\n", herhaal, contactpercentage);
			verbind();
			//printf ("#%i %i%% Reduceren...\n", herhaal, contactpercentage);
			reduceer (contactpercentage);
			//printf ("#%i %i%% Besmetten...\n", herhaal, contactpercentage);
			besmettingsaantal = groeisnelheid(); //besmetten

			printf (">\t%i\t%i\t%i\n", AANTAL, contactpercentage, besmettingsaantal);
			fprintf (fp, "%i\t%i\t%i\n", AANTAL, contactpercentage, besmettingsaantal);

		}

	}

	fclose (fp);

	// laatste graaf uit de herhalingsreeks exporteren
	//printf ("Graphviz...\n");
	//
	// even zeker weten dat er een besmetting zichtbaar is,
	// anders oogt het wel erg saai
	while (aantalbesmet() == 0) besmet_hulp (lijst[rand() % AANTAL]);
	// NOTE: de dubbele verbindingen samenvoegen of de "terugverbinding" verwijderen zou 't netter maken, maar is niet per se nodig
	dot();
	
	printf ("Uitkomsten opgeslagen in: %s\n", TSVFILE);
	printf ("Graphviz opgeslagen in:   %s\n", DOTFILE);

	return 0;
}
